//
// Created by fluffymoo on 11.6.18.
//

#ifndef INC_4NEURO_NEURONBINARY_H
#define INC_4NEURO_NEURONBINARY_H

#include "neuron.h"

/**
 *  Binary neuron class - uses unit-step as the activation function
 */
class NeuronBinary:Neuron {
private:

public:

    /**
     * Default constructor for the binary Neuron
     * @param threshold Denotes when the neuron is activated
     * When neuron potential exceeds 'threshold' value it becomes excited
     */
    explicit NeuronBinary(double threshold = 0.0);

    /**
     * Performs the activation function and stores the result into the 'state' property
     */
    void activate() override;


};


#endif //INC_4NEURO_NEURONBINARY_H

/*
  !-----------------------!----------------------------------------------------------------------
    ! class binary_neuron_t !
    !-----------------------!

    !> Binary neuron class - uses unit-step as the activation function
    type, extends(neuron_t) :: binary_neuron_t
        private

        real(kind=real_4neuro) :: threshold  !< When neuron potential exceeds this value, neuron becomes excited

    contains

        !> Activation function - transforms potential into the output state value
        !! AND assigns it into the 'state' component
        !!
        !! Unit-step function - (returns 1 for potential > threshold, otherwise returns 0)
        procedure, private :: activate => unit_step_activate_impl

    end type binary_neuron_t

    interface binary_neuron_t
        !> Non-parametric constructor of binary_neuron_t class (threshold
        !! will be initialized by a random number from Gaussian distribution)
        !! @return An instance of the class binary_neuron_t
        module procedure :: new_binary_neuron

        !> Parametric constructor of binary_neuron_t class
        !! @param[in] threshold Threshold for the unit-step activation function
        !! @return An instance of the class binary_neuron_t
        module procedure :: new_binary_neuron_1
    end interface binary_neuron_t

    */